/*
 * Copyright (c) 2017. Developed by Hedgecode.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.hedgecode.snooker.json;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

import org.hedgecode.snooker.api.Event;
import org.hedgecode.snooker.api.Events;
import org.hedgecode.snooker.api.Season;
import org.hedgecode.snooker.compare.EventComparators;

/**
 * Events Entity (set of {@link Event}) to JSON deserialize.
 *
 * @author Dmitry Samoshin aka gotty
 */
public class JsonEvents extends JsonCollectionEntity<Event> implements Events {

    private static final JsonEvents EMPTY = new JsonEvents(new Event[0]);

    protected JsonEvents(Event[] entities) {
        super(entities);
    }

    protected JsonEvents(List<Event> entities) {
        super(entities);
    }

    @Override
    public Events bySeason(Season season) {
        List<Event> events = new ArrayList<>();
        for (Event event : this)
            if (event.season().equals(season))
                events.add(event);
        return events.isEmpty() ? EMPTY : new JsonEvents(events);
    }

    @Override
    public Events byType(String type) {
        List<Event> events = new ArrayList<>();
        for (Event event : this)
            if (event.type().equals(type))
                events.add(event);
        return events.isEmpty() ? EMPTY : new JsonEvents(events);
    }

    @Override
    public Events byCity(String city) {
        List<Event> events = new ArrayList<>();
        for (Event event : this)
            if (event.city().equals(city))
                events.add(event);
        return events.isEmpty() ? EMPTY : new JsonEvents(events);
    }

    @Override
    public Events byCountry(String country) {
        List<Event> events = new ArrayList<>();
        for (Event event : this)
            if (event.country().equals(country))
                events.add(event);
        return events.isEmpty() ? EMPTY : new JsonEvents(events);
    }

    @Override
    public Events bySex(String sex) {
        List<Event> events = new ArrayList<>();
        for (Event event : this)
            if (event.sex().equals(sex))
                events.add(event);
        return events.isEmpty() ? EMPTY : new JsonEvents(events);
    }

    @Override
    public Events byStage(String stage) {
        List<Event> events = new ArrayList<>();
        for (Event event : this)
            if (event.stage().equals(stage))
                events.add(event);
        return events.isEmpty() ? EMPTY : new JsonEvents(events);
    }

    @Override
    public Events byValueType(String valueType) {
        List<Event> events = new ArrayList<>();
        for (Event event : this)
            if (event.valueType().equals(valueType))
                events.add(event);
        return events.isEmpty() ? EMPTY : new JsonEvents(events);
    }

    @Override
    public Events bySnookerId(int worldSnookerId) {
        List<Event> events = new ArrayList<>();
        for (Event event : this)
            if (event.worldSnookerId() == worldSnookerId)
                events.add(event);
        return events.isEmpty() ? EMPTY : new JsonEvents(events);
    }

    @Override
    public Events ended() {
        List<Event> events = new ArrayList<>();
        long currentTime = System.currentTimeMillis();
        for (Event event : this)
            if (currentTime > nextDate(event.endDate()).getTime())
                events.add(event);
        return events.isEmpty() ? EMPTY : new JsonEvents(events);
    }

    @Override
    public Events current() {
        List<Event> events = new ArrayList<>();
        long currentTime = System.currentTimeMillis();
        for (Event event : this)
            if (currentTime > event.startDate().getTime()
                    && currentTime < nextDate(event.endDate()).getTime())
                events.add(event);
        return events.isEmpty() ? EMPTY : new JsonEvents(events);
    }

    @Override
    public Events upcoming() {
        List<Event> events = new ArrayList<>();
        long currentTime = System.currentTimeMillis();
        for (Event event : this)
            if (currentTime < event.startDate().getTime())
                events.add(event);
        return events.isEmpty() ? EMPTY : new JsonEvents(events);
    }

    @Override
    public void sortByDate() {
        sort(
                EventComparators.DATE.comparator()
        );
    }

    @Override
    public void sortBySnookerId() {
        sort(
                EventComparators.SNOOKER_ID.comparator()
        );
    }

    private Date nextDate(Date date) {
        Calendar c = Calendar.getInstance();
        c.setTime(date);
        c.add(Calendar.DATE, 1);
        return c.getTime();
    }

}
